import 'dart:async';
import 'dart:isolate';

typedef FutureOr<void> EntryPoint(CrossCommunicationChannel channel);

class _IsolateParameterData {
  final SendPort sendPort;
  final EntryPoint entryPoint;

  _IsolateParameterData(this.sendPort, this.entryPoint);
}

void _isolateMain(_IsolateParameterData parameterData) async {
  CrossCommunicationChannel channel = CrossCommunicationChannel();
  channel.setupSendPort(parameterData.sendPort);

  // Send the isolate send port
  channel.send(channel.getSendPortForOther());

  await parameterData.entryPoint(channel);
}

class CrossCommunicationChannel {
  final ReceivePort _receivePort;
  StreamIterator<dynamic> _receiveIterator;
  SendPort _sendPort;

  CrossCommunicationChannel() : _receivePort = ReceivePort() {
    _receiveIterator = StreamIterator(_receivePort);
  }

  SendPort getSendPortForOther() => _receivePort.sendPort;

  Future<dynamic> next() async {
    await _receiveIterator.moveNext();
    return _receiveIterator.current;
  }

  void send(message) => _sendPort.send(message);

  void setupSendPort(SendPort port) => _sendPort = port;
}

class CrossCommunicationIsolate {
  CrossCommunicationChannel _masterChannel;
  Future<void> _startup;
  Isolate _isolate;

  CrossCommunicationIsolate() : _masterChannel = CrossCommunicationChannel();

  Isolate get isolate => _isolate;

  Future<void> start(EntryPoint entryPoint) async {
    Completer startupCompleter = Completer();
    _startup = startupCompleter.future;

    SendPort other = _masterChannel.getSendPortForOther();
    _isolate = await Isolate.spawn(
        _isolateMain, _IsolateParameterData(other, entryPoint));

    SendPort here = await _masterChannel.next();
    _masterChannel.setupSendPort(here);

    startupCompleter.complete();
  }

  Future<void> waitForStartup() => _startup;

  Future<dynamic> next() => _masterChannel.next();

  void send(message) => _masterChannel.send(message);
}
