import 'package:libutil/ansi.dart';

class LogLevel {
  static const LogLevel VERBOSE = LogLevel._(0, 'verbose', Ansi(fg: Ansi.GRAY));
  static const LogLevel DEBUG = LogLevel._(1, 'debug', Ansi(fg: Ansi.GREEN));
  static const LogLevel INFO = LogLevel._(2, 'info', Ansi());
  static const LogLevel WARNING =
      LogLevel._(3, 'warning', Ansi(fg: Ansi.BLACK, bg: Ansi.YELLOW));
  static const LogLevel ERROR =
      LogLevel._(4, 'error', Ansi(fg: Ansi.WHITE, bg: Ansi.RED));

  final int index;
  final String name;
  final Ansi color;

  const LogLevel._(this.index, this.name, this.color);
}

class Logger {
  final String name;

  Logger(this.name);

  void log(LogLevel level, dynamic message) {
    if (message is Function) {
      log(level, message());
    } else {
      print(
          '${level.color} $name/${level.name.toUpperCase()}: $message ${Ansi.RESET}');
    }
  }

  void verbose(message) => log(LogLevel.VERBOSE, message);

  void debug(message) => log(LogLevel.DEBUG, message);

  void info(message) => log(LogLevel.INFO, message);

  void warn(message) => log(LogLevel.WARNING, message);

  void error(message) => log(LogLevel.ERROR, message);
}
